#!/usr/bin/perl
#
#    Quality of Service Webmin Module
#    Copyright (C) 2001 by Level 7 Software
#
#    Author: Dan Damian <dand@level7.ro>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

do '../web-lib.pl';
require '../net/net-lib.pl';

&init_config();

$CONF_DIR = $config{'conf_dir'};

$header_note = "<font color=#c00000><b>WARNING:<br>Alpha version!</b></font><br>";
$website_link = "<a href=\"http://webmin-cbq.sourceforge.net\">Module Website</a>";

%access = &get_module_acl();

# active_interfaces()
# Returns a list of current ip links
sub active_interfaces
{
    my (@rv, @lines, $l);
    open(IFC, "ip link |");
    while(<IFC>) {
	s/\r|\n//g;
	if (/^\S+/) { push(@lines, $_); }
	else { $lines[$#lines] .= $_; }
    }    
    close(IFC);

    foreach $l (@lines) {
	my ($ifname);
	$l =~ /^*.: (\S+)/; 
	$ifname = $1;
	$ifname =~ s/://;
	push(@rv, $ifname);
    }
    
    return @rv;
}

sub parse_config_dir 
{
    local $f;
    my @rv;
    
    opendir(CONF, $CONF_DIR);
    while ($f = readdir(CONF)) {
	next if ($f !~ /^cbq-[0-9]+\.\S+$/);
	$f = "$CONF_DIR/$f";
	my (%conf, %c);
	&read_config_file("$f", \%conf);
	&parse_config_file (\%conf, \%c);
	push(@rv, \%c);
    }
    closedir(CONF);
    return @rv;
}

# get_file (classid)
sub get_file
{
    my ($rv, $f);
    
    opendir(CONF, $CONF_DIR);
    while ($f = readdir(CONF)) {
	next if ($f !~ /^cbq-$_[0]\.\S+$/);
	$rv = "$CONF_DIR/$f";
	break;
    }
    closedir(CONF);
    return $rv;
}


sub load_class
{
    my ($classid) = (@_[0]);
    my (%rv, %rv1, %conf);

    local $f = get_file($classid);

    &read_config_file($f, \%conf) || return 0;
    &parse_config_file(\%conf, \%rv);

    if (!%rv) {
	print "ERROR: Could not load class $f<p>";
	&footer("index.cgi", "Index");
	exit;
    }
    
   return \%rv;
}

sub parse_config_file
{
    my ($conf, $c) = ($_[0], $_[1]);

    $c->{'device'} = $conf->{'DEVICE'};
    if ($c->{'device'} =~ /(\S+),(\S*),(\S*)/) {
	$c->{'ifname'} = $1;
	$c->{'bandwidth'} = $2;
	$c->{'bweight'} = $3;
    } else {
	$c->{'ifname'} = $c->{'device'}
    }
    
    # Class parameters
    if ($f =~ /^.*\/cbq-([0-9]+)\.(\S+)/) {
	$c->{'classid'} = $1;
	$c->{'classname'} = $2;
    }
    $c->{'rate'}     = $conf->{'RATE'};
    $c->{'weight'}   = $conf->{'WEIGHT'};
    $c->{'prio'}     = $conf->{'PRIO'};
    $c->{'parent'}   = $conf->{'PARENT'};
    $c->{'leaf'}     = $conf->{'LEAF'};
    $c->{'bounded'}  = $conf->{'BOUNDED'};
    $c->{'isolated'} = $conf->{'ISOLATED'};
    
    # Filter parameters
    $c->{'rules'} = $conf->{'RULE'};
    $c->{'inactive_rules'} = $conf->{'INACTIVE_RULE'};
    
    # Time ranging parameters
    $c->{'time'} = $conf->{'TIME'};

#    print "D: " . $_[1]->{'device'}. "<br>";
    
    return 1;
}

# read_config_file(file, &array)
sub read_config_file
{
    my (@rules, @i_rules, $val);
    open(FILE, $_[0]) || return 0;
    while(<FILE>) {
	if (/(RULE)\s*=\s*(\S*)\s*$/) {
	    $val = $2;
	    /\#/ ? push(@i_rules, $val) : push(@rules, $val);
	} else {
	    s/#.*$//g;
	    if (/([A-z0-9_]+)\s*=\s*"(\S*)"/ ||
		/([A-z0-9_]+)\s*=\s*'(\S*)'/ ||
		/([A-z0-9_]+)\s*=\s*(\S*)/) {
		$_[1]->{$1} = $2;		
#		print "Got: $1 = '$2'<br>";
	    }
	}
    }
    $_[1]->{'RULE'} = \@rules;
    $_[1]->{'INACTIVE_RULE'} = \@i_rules;
    close(FILE);
    return 1;
}

sub rules_list
{
    my (@rule_list, $rule);
    $rule_list = $c->{$_[0]};
    foreach $rule (@$rule_list) {
	print "<option value=\"$rule\">$rule";
    }
}

sub select_list # (default_value, option1, option2, ...)
{
    for ($i=1; $i<=$#_; $i++) {
	print "<option value=\"$_[$i]\"",
	$_[$i] eq $_[0] ? " selected" : "",
	">$_[$i]</option>\n";
    }
}

# diffread_config_file(file)
sub diffread_config_file
{
    my ($val, $orig, %old_c, $line, $skip_line);
    open(FILE, $_[0]) || return 0;
    while(<FILE>) {
	$skip_line = 0;
	$line = $_;
	if (/(RULE)\s*=\s*(\S*)\s*$/) {
	    $val = $2;
	    $orig = $1;
	    #/\#/ ? push(@i_rules, $val) : push(@rules, $val);
	    if ($in{'action'} eq "activate" && $val eq $in{'irselect'}) {
		$line =~ s/\#\s*$orig/$orig/;
	    } else {
		if ($in{'action'} eq "inactivate" && $val eq $in{'rselect'}) {
		    $line =~ s/$orig/\# $orig/;
		} else {
		    if ($in{'action'} eq "delete" && $val eq $in{'rdelete'}) {
			$skip_line = 1;
		    }
		}
	    };
	    #print /\#/ ? "A: $val<br>" : "I: $val | $in{'rselect'}<br>";
	} else {
	    if ($in{'action'} eq "save") {
		s/#.*$//g;
		if (/([A-z0-9_]+)\s*=\s*"(\S*)"/ ||
		    /([A-z0-9_]+)\s*=\s*'(\S*)'/ ||
		    /([A-z0-9_]+)\s*=\s*(\S*)/) {
		    $key = lc($1);
		    $val = $2;
		  SW: for ($key) {
		      /bounded/ | /isolated/ && do {
			  $line =~ s/yes/no/ if ($in{$key} ne "on"  && $val eq "yes");
			  $line =~ s/no/yes/ if ($in{$key} eq "on"  && $val eq "no");
			  last;
		      };
		      /device/ && do {		      
			  if ($val =~ /(\S+),(\S*),(\S*)/) {
			      $old_c{'ifname'} = $1;
			      $old_c{'bandwidth'} = $2;
			      if ($in{'ifname'} ne $1 ||
				  $in{'bandwidth'} ne $2) {
				  $newdev = "$in{'ifname'},$in{'bandwidth'},". $in{'bandwidth'} / 10 . "Mbit";
				  $line =~ s/$val/$newdev/;
				  print "$val / $newdev";
			      }
			  } else {
			      $old_c->{'ifname'} = $val;
			  }			  		      
			  last;
		      };
		      /weight/ && do {			  
			  $newval = $in{'rate'} / 10;
			  $line =~ s/$val/$newval/;
			  last;
		      }
		  }
		    #print "O: $line<br>";
		    $line =~ s/$val/$in{$key}/ if ($in{$key} != $val && defined($in{$key}));
		    #print "$1: [$in{$key}]=[$val]<br>";
		    #print "N: $line<br>";
		    $old_c{$key} = $val;	       
		}
	    }
	}
	push(@file_cache, $line) if (!$skip_line);
    }
    close(FILE);

    if ($in{'action'} eq "save") {
       # Class parameters
	if ($f =~ /^.*\/cbq-([0-9]+)\.(\S+)/) {
	    $old_c{'classid'} = $1;
	    $old_c{'classname'} = $2;
	}
	
	if ($in{'classid'} ne $old_c{'classid'} || $in{'classname'} ne $old_c{'classname'}) {
	    rename($f, "$CONF_DIR/cbq-$in{'classid'}.$in{'classname'}");
	    $f = "$CONF_DIR/cbq-$in{'classid'}.$in{'classname'}";
	}
	
	push(@file_cache, "BOUNDED=yes\n") if (!defined($old_c{'bounded'}) && $in{'bounded'} eq "on");
	push(@file_cache, "ISOLATED=yes\n") if (!defined($old_c{'isolated'}) && $in{'isolated'} eq "on");
	push(@file_cache, "PRIO=$in{'prio'}\n") if (!defined($old_c{'prio'}));
	push(@file_cache, "PARENT=$in{'parent'}\n") if (!defined($old_c{'parent'}) && $in{'parent'} ne "");
	push(@file_cache, "RATE=$in{'rate'}\n") if (!defined($old_c{'rate'}) && $in{'rate'} ne "");
	push(@file_cache, "WEIGHT=" . $in{'rate'} / 10 . "\n") if (!defined($old_c{'weight'}));
								   
	if ((!defined($old_c{'ifname'}) && $in{'ifname'} ne "") ||
	    (!defined($old_c{'bandwidth'}) && $in{'bandwidth'} ne "")){
	    $newdev = "$in{'ifname'},$in{'bandwidth'},". $in{'bandwidth'} / 10 . "Mbit";
	    unshift(@file_cache, "DEVICE=$newdev\n");
	}
    }

    flush_config_file($_[0]);
    
    return 1;
}

# flush_config_file(file)
sub flush_config_file
{
    &lock_file($_[0]);
    open(FLUSHFILE, "> $_[0]");
    foreach $line (@file_cache) {
	print FLUSHFILE $line;
    }
    close(FLUSHFILE);
    &unlock_file($_[0]);
}                                       

sub add_rule
{
    return if (!defined($in{'rnew'}));
    &lock_file($_[0]);    
    open(FILE, ">> $_[0]");
    print FILE "RULE=$in{'rnew'}\n";
    close(FILE);
    &unlock_file($_[0]);    
}

sub save_class
{    
    local $f = get_file($in{'old_classid'});
    
    $in{action} =~ /^activate/  ||  $in{action} =~ /inactivate/ || $in{action} =~ /save/ || $in{action} =~ /delete/
	?  &diffread_config_file($f) :
	    $in{action} =~ /^add$/  ? &add_rule($f) : "";
    redirect("index.cgi") if ($in{action} eq "save");
}

sub new_class
{
    my ($f);
    $f = "$CONF_DIR/cbq-$in{'classid'}.$in{'classname'}";
    `touch $f`;
}

sub delete_class
{
    my ($f);
    $f = "$CONF_DIR/cbq-$c->{'classid'}.$c->{'classname'}";
    unlink($f);
    redirect("index.cgi");
}

1;
